/*
***************************************************************************
*
* Author: Teunis van Beelen
*
* Copyright (C) 2011 Teunis van Beelen
*
* teuniz@gmail.com
*
***************************************************************************
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation version 2 of the License.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
***************************************************************************
*
* This version of GPL is at http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt
*
***************************************************************************
*/



#include "averager_curve_wnd.h"




UI_AverageCurveWindow::UI_AverageCurveWindow(struct signalcompblock *signal_comp, QWidget *parent, int number)
{
  char str[1024];


  averager_curve_dialog_is_destroyed = 0;

  mainwindow = (UI_Mainwindow *)parent;

  signalcomp = signal_comp;

  averagecurvedialognumber = number;

  pagetime = signalcomp->avg_pagetime;

  pagetime /= TIME_DIMENSION;

  averager_curve_dialog = new QDialog();
  averager_curve_dialog->setAttribute(Qt::WA_DeleteOnClose, TRUE);
  averager_curve_dialog->setMinimumSize(QSize(650, 480));
  averager_curve_dialog->setSizeGripEnabled(TRUE);
  averager_curve_dialog->setModal(FALSE);
  averager_curve_dialog->setWindowFlags(Qt::Window | Qt::WindowMinMaxButtonsHint | Qt::WindowCloseButtonHint);
  averager_curve_dialog->setWindowIcon(QIcon(":/images/edf.png"));

  strcpy(str, "Averaging  ");
  strcat(str, signalcomp->signallabel);
  averager_curve_dialog->setWindowTitle(str);

  curve1 = new SignalCurve;
  curve1->setSignalColor(Qt::green);
  curve1->setBackgroundColor(Qt::black);
  curve1->setRasterColor(Qt::gray);
  curve1->setTraceWidth(0);
  curve1->setH_RasterSize(10);
  curve1->setV_RasterSize(10);
  curve1->setH_label("sec");
  curve1->setLowerLabel("Time");
  curve1->setV_label(signalcomp->physdimension);
  curve1->setDashBoardEnabled(FALSE);
  curve1->setMarker1Color(Qt::yellow);

  sprintf(str, "Averaging %i triggers \"%s\"", signalcomp->avg_cnt, signalcomp->avg_annotation);
  curve1->setUpperLabel1(str);

  amplitudeSlider = new QSlider;
  amplitudeSlider->setOrientation(Qt::Vertical);
  amplitudeSlider->setMinimum(1);
  amplitudeSlider->setMaximum(2000);
  amplitudeSlider->setValue(1000);
  amplitudeSlider->setInvertedAppearance(TRUE);
  amplitudeSlider->setMinimumSize(15, 280);

  amplitudeLabel = new QLabel;
  amplitudeLabel->setText("Amplitude");
  amplitudeLabel->setMinimumSize(110, 15);
  amplitudeLabel->setAlignment(Qt::AlignHCenter);

  vlayout2 = new QVBoxLayout;
  vlayout2->setSpacing(20);
  vlayout2->addStretch(100);
  vlayout2->addWidget(amplitudeLabel, 0, Qt::AlignHCenter);
  vlayout2->addWidget(amplitudeSlider, 0, Qt::AlignHCenter);

  spanSlider = new QSlider;
  spanSlider->setOrientation(Qt::Horizontal);
  spanSlider->setMinimum(10);
  spanSlider->setMaximum(1000);
  spanSlider->setValue(1000);
  spanSlider->setMinimumSize(500, 15);

  spanLabel = new QLabel;
  spanLabel->setText("Span");
  spanLabel->setMinimumSize(110, 15);
  spanLabel->setAlignment(Qt::AlignHCenter);

  centerSlider = new QSlider;
  centerSlider->setOrientation(Qt::Horizontal);
  centerSlider->setMinimum(0);
  centerSlider->setMaximum(1000);
  centerSlider->setValue(250);
  centerSlider->setMinimumSize(500, 15);

  centerLabel = new QLabel;
  centerLabel->setText("Center");
  centerLabel->setMinimumSize(110, 15);
  centerLabel->setAlignment(Qt::AlignHCenter);

  hlayout1 = new QHBoxLayout;
  hlayout1->setSpacing(20);
  hlayout1->addLayout(vlayout2);
  hlayout1->addWidget(curve1, 100);

  hlayout2 = new QHBoxLayout;
  hlayout2->setSpacing(20);
  hlayout2->addWidget(spanLabel);
  hlayout2->addWidget(spanSlider);
  hlayout2->addStretch(100);

  hlayout3 = new QHBoxLayout;
  hlayout3->setSpacing(20);
  hlayout3->addWidget(centerLabel);
  hlayout3->addWidget(centerSlider);
  hlayout3->addStretch(100);

  vlayout1 = new QVBoxLayout;
  vlayout1->setSpacing(20);
  vlayout1->addLayout(hlayout1);
  vlayout1->addLayout(hlayout2);
  vlayout1->addLayout(hlayout3);

  averager_curve_dialog->setLayout(vlayout1);

  QObject::connect(amplitudeSlider,       SIGNAL(valueChanged(int)),    this, SLOT(sliderMoved(int)));
  QObject::connect(spanSlider,            SIGNAL(valueChanged(int)),    this, SLOT(sliderMoved(int)));
  QObject::connect(centerSlider,          SIGNAL(valueChanged(int)),    this, SLOT(sliderMoved(int)));
  QObject::connect(averager_curve_dialog, SIGNAL(destroyed(QObject *)), this, SLOT(averager_curve_dialogDestroyed(QObject *)));

  sliderMoved(0);

  averager_curve_dialog->show();
}


void UI_AverageCurveWindow::sliderMoved(int)
{
  int startstep,
      stopstep,
      precision,
      spanstep,
      steps,
      markerstep;

  double max_sec,
         start_sec,
         dtmp;


  steps = signalcomp->avg_samples_on_screen;

  spanstep = spanSlider->value() * steps / 1000;

  startstep = centerSlider->value() * (steps - spanstep) / 1000;

  stopstep = startstep + spanstep;

  dtmp = signalcomp->avg_max_value * (double)amplitudeSlider->value() / 1000.0;

  curve1->drawCurve(signalcomp->avgbuf + startstep, stopstep - startstep, dtmp, -dtmp);

  max_sec = pagetime * stopstep / steps - (pagetime * (1.0 / signalcomp->avg_trigger_position_ratio));

  precision = 0;
  if(max_sec < 1000.0)
  {
    precision = 1;
  }
  if(max_sec < 100.0)
  {
    precision = 2;
  }
  if(max_sec < 10.0)
  {
    precision = 3;
  }
  if(max_sec < 1.0)
  {
    precision = 4;
  }

  start_sec = pagetime * startstep / steps - (pagetime * (1.0 / signalcomp->avg_trigger_position_ratio));

  curve1->setH_RulerValues(start_sec, max_sec, precision);

  centerLabel->setText(QString::number(start_sec + ((max_sec - start_sec) / 2.0), 'f', precision).append(" sec").prepend("Center "));

  spanLabel->setText(QString::number(max_sec - start_sec, 'f', precision).append(" sec").prepend("Span "));

  if(signalcomp->avg_trigger_position_ratio > 0)
  {
    markerstep = steps / signalcomp->avg_trigger_position_ratio;

    if((markerstep > startstep) && (markerstep < stopstep))
    {
      curve1->setMarker1Enabled(TRUE);

      curve1->setMarker1Position((double)(markerstep - startstep) / (double)spanstep);
    }
    else
    {
      curve1->setMarker1Enabled(FALSE);
    }
  }
  else
  {
    curve1->setMarker1Enabled(FALSE);
  }
}


void UI_AverageCurveWindow::averager_curve_dialogDestroyed(QObject *)
{
  averager_curve_dialog_is_destroyed = 1;

  mainwindow->averagecurvedialog[averagecurvedialognumber] = NULL;

  delete this;
}


UI_AverageCurveWindow::~UI_AverageCurveWindow()
{
  if(!averager_curve_dialog_is_destroyed)
  {
    averager_curve_dialog->close();
  }

  if(signalcomp->avgbuf != NULL)
  {
    free(signalcomp->avgbuf);

    signalcomp->avgbuf = NULL;
  }

  mainwindow->averagecurvedialog[averagecurvedialognumber] = NULL;
}



























